@@ -4,12 +4,14 @@ import math
 import drawSvg as draw
 
 from typing import Optional
-from track_generator.track import Track, Start, Straight, Arc, Crosswalk, Intersection, Gap, ParkingArea, TrafficIsland
+from track_generator.track import Track, Start, Straight, Arc, Crosswalk, Intersection, Gap, ParkingArea, TrafficIsland, Clothoid
 from track_generator.coordinate_system import Point2d, Polygon
 
 DEFAULT_LINE_WIDTH = 0.020
 DEFAULT_TRACK_WIDTH = 0.800
 
+DEFAULT_TRACK_COLOR = '#000000'
+DEFAULT_LINE_COLOR = '#ffffff'
 
 class Painter:
 
@@ -28,7 +30,7 @@ class Painter:
         self.d.append(draw.Circle(p.x_w,
                                   p.y_w,
                                   0.010,
-                                  fill='red', stroke_width=0, stroke='black'))
+                                  fill='red', stroke_width=0, stroke=DEFAULT_TRACK_COLOR))
 
         self.d.append(draw.Text(f'{p.x_w:.3f}\n{p.y_w:.3f}', 0.1, p.x_w + 0.032, p.y_w, fill='red'))
 
@@ -36,7 +38,7 @@ class Painter:
         self.d.append(draw.Circle(p.x_w,
                                   p.y_w,
                                   0.010,
-                                  fill='red', stroke_width=0, stroke='black'))
+                                  fill='red', stroke_width=0, stroke=DEFAULT_TRACK_COLOR))
 
         self.d.append(draw.Text(f'{p.x_w:.3f}\n{p.y_w:.3f}\nr={radius:.3f}\na={radian_angle:.1f}°', 0.1, p.x_w + 0.032, p.y_w, fill='red'))
 
@@ -45,13 +47,13 @@ class Painter:
 
     def draw_straight(self, segment: Straight):
 
-        self.draw_polygon(segment.center_line_polygon, fill='#eeee00', stroke='black', stroke_width=DEFAULT_TRACK_WIDTH)
-        self.draw_polygon(segment.center_line_polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none',
-                            style="stroke-miterlimit:4;stroke-dasharray:0.16,0.16;stroke-dashoffset:0")
+        self.draw_polygon(segment.center_line_polygon, fill=DEFAULT_TRACK_COLOR, stroke=DEFAULT_TRACK_COLOR, stroke_width=DEFAULT_TRACK_WIDTH)
+        self.draw_polygon(segment.center_line_polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none',
+                            style="stroke-miterlimit:4;stroke-dasharray:0.16,0.16;stroke-dashoffset:0,fill-opacity:0")
         # stroke-miterlimit:4;stroke-dasharray:0.08,0.16;stroke-dashoffset:0;stroke-width:0.02
 
-        self.draw_polygon(segment.left_line_polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
-        self.draw_polygon(segment.right_line_polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+        self.draw_polygon(segment.left_line_polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+        self.draw_polygon(segment.right_line_polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
     def draw_straight_verbose(self, segment: Straight):
         self.draw_point(segment.center_line_polygon[0])
@@ -70,16 +72,16 @@ class Painter:
             final_start_angle = start_angle - 90
 
         self.d.append(draw.Arc(segment.center_point.x_w, segment.center_point.y_w, math.fabs(segment.radius), final_start_angle, final_end_angle,
-                               cw=segment.direction_clockwise, stroke='black', stroke_width=DEFAULT_TRACK_WIDTH, fill='none'))
+                               cw=segment.direction_clockwise, stroke=DEFAULT_TRACK_COLOR, stroke_width=DEFAULT_TRACK_WIDTH, fill='none'))
 
         self.d.append(draw.Arc(segment.center_point.x_w, segment.center_point.y_w, math.fabs(segment.radius) - 0.380, final_start_angle,
-                               final_end_angle, cw=segment.direction_clockwise, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none'))
+                               final_end_angle, cw=segment.direction_clockwise, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none'))
 
         self.d.append(draw.Arc(segment.center_point.x_w, segment.center_point.y_w, math.fabs(segment.radius) + 0.380, final_start_angle,
-                               final_end_angle, cw=segment.direction_clockwise, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none'))
+                               final_end_angle, cw=segment.direction_clockwise, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none'))
 
         self.d.append(draw.Arc(segment.center_point.x_w, segment.center_point.y_w, math.fabs(segment.radius), final_start_angle, final_end_angle,
-                               cw=segment.direction_clockwise, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none',
+                               cw=segment.direction_clockwise, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none',
                                style="stroke-miterlimit:4;stroke-dasharray:0.160,0.160;stroke-dashoffset:0"))
 
     def draw_arc_verbose(self, segment: Arc):
@@ -105,26 +107,26 @@ class Painter:
         self.draw_point(segment.start_point_right)
 
     def draw_crosswalk(self, segment: Crosswalk):
-        self.draw_polygon(segment.center_line_polygon, fill='#eeee00', stroke='black', stroke_width=DEFAULT_TRACK_WIDTH)
-        self.draw_polygon(segment.left_line_polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
-        self.draw_polygon(segment.right_line_polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+        self.draw_polygon(segment.center_line_polygon, fill='#eeee00', stroke=DEFAULT_TRACK_COLOR, stroke_width=DEFAULT_TRACK_WIDTH)
+        self.draw_polygon(segment.left_line_polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+        self.draw_polygon(segment.right_line_polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
         for polygon in segment.line_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=0.03, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=0.03, fill='none')
 
     def draw_intersection(self, segment: Intersection):
 
         for polygon in segment.base_line_polygons:
-            self.draw_polygon(polygon, fill='#eeee00', stroke='black', stroke_width=DEFAULT_TRACK_WIDTH)
+            self.draw_polygon(polygon, fill='#eeee00', stroke=DEFAULT_TRACK_COLOR, stroke_width=DEFAULT_TRACK_WIDTH)
 
         for polygon in segment.corner_line_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
         for polygon in segment.stop_line_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH*2, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH*2, fill='none')
 
         for polygon in segment.center_line_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none',
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none',
                             style="stroke-miterlimit:4;stroke-dasharray:0.16,0.16;stroke-dashoffset:0")
 
     def draw_traffic_island(self, segment: TrafficIsland):
@@ -138,13 +140,13 @@ class Painter:
             segment.background_polygon[6].x_w, segment.background_polygon[6].y_w,
             segment.background_polygon[7].x_w, segment.background_polygon[7].y_w,
             close=False,
-            fill='black'))
+            fill=DEFAULT_TRACK_COLOR))
 
         for polygon in segment.line_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
         for polygon in segment.crosswalk_lines_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=0.03, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=0.03, fill='none')
 
     def draw_parking_area(self, segment: ParkingArea):
         self.draw_straight(segment)
@@ -157,22 +159,41 @@ class Painter:
                 polygon[2].x_w, polygon[2].y_w,
                 polygon[3].x_w, polygon[3].y_w,
                 close=False,
-                fill='black'))
+                fill=DEFAULT_TRACK_COLOR))
             # Outline
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
         for polygon in segment.spot_seperator_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
         for polygon in segment.blocker_polygons:
-            self.draw_polygon(polygon, stroke='white', stroke_width=DEFAULT_LINE_WIDTH, fill='none')
+            self.draw_polygon(polygon, stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH, fill='none')
 
+    def draw_clothoid(self, segment: Clothoid):
+        background: tuple(float) = ()
+        for p in segment.background_polygon:
+            background += (p.x_w, p.y_w)
+        self.d.append(draw.Lines(background[0], background[1], *background, fill=DEFAULT_TRACK_COLOR, stroke=DEFAULT_TRACK_COLOR, stroke_width=2*DEFAULT_LINE_WIDTH))
+        middle_line: tuple(float) = ()
+        for p in segment.lines[0]:
+            middle_line += (p.x_w, p.y_w)
+        self.d.append(draw.Lines(*middle_line, fill='none', stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH
+                        ,style="stroke-miterlimit:4;stroke-dasharray:0.16,0.16;stroke-dashoffset:0"))
+        left_line: tuple(float) = ()
+        for p in segment.lines[1]:
+            left_line += (p.x_w, p.y_w)
+        self.d.append(draw.Lines(*left_line, fill='none', stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH))
+        right_line: tuple(float) = ()
+        for p in segment.lines[2]:
+            right_line += (p.x_w, p.y_w)
+        self.d.append(draw.Lines(*right_line, fill='none', stroke=DEFAULT_LINE_COLOR, stroke_width=DEFAULT_LINE_WIDTH))
+    
     def draw_template_based_segment(self, segment, template_file_path: str):
         self.d.append(draw.Image(segment.start_point_center.x_w - (segment.width / 2.0), segment.start_point_center.y_w, segment.width, segment.height,
                                  template_file_path,
                                  embed=True,
                                  transform=f'rotate({-(segment.direction_angle - 90.0)} , {segment.start_point_center.x_w}, {-segment.start_point_center.y_w})'))
-
+    
     def draw_segment(self, segment):
         if isinstance(segment, Start):
             pass
@@ -190,6 +211,8 @@ class Painter:
             self.draw_intersection(segment)
         elif isinstance(segment, TrafficIsland):
             self.draw_traffic_island(segment)
+        elif isinstance(segment, Clothoid):
+            self.draw_clothoid(segment)
         else:
             raise RuntimeError()
 
@@ -204,8 +227,17 @@ class Painter:
     def draw_track(self, track: Track):
         self.d = draw.Drawing(track.width, track.height, origin=track.origin, displayInline=False)
         self.d.setPixelScale(1000)
-        self.d.append(draw.Rectangle(0, 0, track.width, track.height, fill=track.background_color,
-                                     fill_opacity=track.background_opacity))
+        self.d.append(draw.Rectangle(0, 0, track.width, track.height, fill=track.background.color.color,
+                                     fill_opacity=track.background.color.opacity))
+
+        if track.background.image:
+            img = track.background.image
+            self.d.append(draw.Image(img.x, img.y,
+                                     img.width, img.height,
+                                     img.file,
+                                     embed=True,
+                                     preserveAspectRatio='none'))
+
         for segment in track.segments:
             self.draw_segment(segment)
 
