@@ -5,6 +5,7 @@ import sys
 
 from track_generator import generator
 
+
 VERSION = '0.0.1'
 
 
@@ -18,9 +19,22 @@ def subcommand_generate_track():
                         help='Generate output PNG (SVG only by default).')
     parser.add_argument('--gazebo', action='store_true',
                         help='Generate Gazebo model for track.')
+    parser.add_argument('--ground_truth', action='store_true',
+                        help='Generate ground truth data for track.')
     args = parser.parse_args(sys.argv[2:])
 
-    generator.generate_track(args.track_files, args.output, args.png, args.gazebo)
+    generator.generate_track(args.track_files, args.output, args.png, args.gazebo, args.ground_truth)
+
+
+def subcommand_generate_track_live():
+    parser = argparse.ArgumentParser(description='Generates a track')
+    parser.add_argument('track_file', metavar='track_file', type=str,
+                        help='a track file (XML) to generate the track from')
+    parser.add_argument('-o', '--output', dest='output', default=os.path.join(os.getcwd(), 'output'),
+                        help='Output directory for generated tracks. Default="./"')
+    args = parser.parse_args(sys.argv[2:])
+
+    generator.generate_track_live(args.track_file, args.output)
 
 
 def subcommand_generate_trajectory():
@@ -36,9 +50,12 @@ def start():
 
     if args.command == 'generate_track':
         subcommand_generate_track()
+    elif args.command == 'generate_track_live':
+        subcommand_generate_track_live()
     elif args.command == 'generate_trajectory':
         subcommand_generate_trajectory()
 
 
 if __name__ == '__main__':
     start()
+
