@@ -1,7 +1,7 @@
 # Copyright (C) 2022 twyleg
 import numpy
 from enum import Enum
-from math import tan
+from math import tan, factorial, sqrt, sin, cos, radians, pi
 from typing import Any, List, Tuple, Optional
 from track_generator.coordinate_system import Polygon, Point2d, CartesianSystem2d
 
@@ -18,6 +18,21 @@ class Side(Enum):
     RIGHT = 2
 
 
+class IntersectionDirection(Enum):
+    RIGHT = 'right'
+    LEFT = 'left'
+    STRAIGHT = 'straight'
+
+class ClothoidDirection(Enum):
+    LEFT = -1
+    RIGHT = 1
+
+
+class ClothoidType(Enum):
+    OPEND = 'opend'
+    CLOSING = 'closing'
+
+
 def calc_crosswalk_lines(length: float, width: float, coordinate_system: CartesianSystem2d) -> List[Polygon]:
     polygons: List[Polygon] = []
     pack_width = CROSSWALK_LINE_WIDTH + CROSSWALK_LINE_GAP
@@ -43,15 +58,34 @@ def calc_crosswalk_lines(length: float, width: float, coordinate_system: Cartesi
     return polygons
 
 
+class Background:
+
+    class Color:
+        def __init__(self, color: str, opacity: float):
+            self.color = color
+            self.opacity = opacity
+
+    class Image:
+        def __init__(self, file: str, x: float, y: float, width: float, height: float):
+            self.file = file
+            self.x = x
+            self.y = y
+            self.width = width
+            self.height = height
+
+    def __init__(self, color: Color = None, image: Image = None):
+        self.color: Optional[Background.Color] = color
+        self.image: Optional[Background.Image] = image
+
+
 class Track:
     def __init__(self, version: str, width: float, height: float, origin: Tuple[float, float],
-                 background_color: str, background_opacity: float, segments: List[Any]):
+                 background: Background, segments: List[Any]):
         self.version = version
         self.width = width
         self.height = height
         self.origin = origin
-        self.background_color = background_color
-        self.background_opacity = background_opacity
+        self.background = background
         self.segments = segments
 
     def calc(self) -> None:
@@ -171,9 +205,10 @@ class Crosswalk(Straight):
 
 
 class Intersection(Segment):
-    def __init__(self, length: float):
+    def __init__(self, length: float, direction: IntersectionDirection):
         super().__init__()
         self.length = length
+        self.direction = direction
         self.base_line_polygons: List[Polygon] = []
         self.corner_line_polygons: List[Polygon] = []
         self.stop_line_polygons: List[Polygon] = []
@@ -254,7 +289,14 @@ class Intersection(Segment):
 
     def calc(self, prev_segment) -> None:
         super().calc(prev_segment)
-        self.end_coordinate_system = CartesianSystem2d(self.length, 0.0, 0.0, self.start_coordinate_system)
+        if self.direction == IntersectionDirection.RIGHT:
+            new_end = [self.length/2, -self.length/2, -90]
+        elif self.direction == IntersectionDirection.STRAIGHT:
+            new_end = [self.length, 0, 0]
+        else:
+            new_end = [self.length/2, self.length/2, 90]
+        self.end_coordinate_system = CartesianSystem2d(new_end[0], new_end[1], new_end[2], self.start_coordinate_system)
+        self.direction_angle += new_end[2]
 
         self.calc_base_lines()
         self.calc_corner_lines()
@@ -263,8 +305,21 @@ class Intersection(Segment):
 
 
 class Gap(Straight):
-    def __init__(self, length: float):
+    def __init__(self, length: float, direction: IntersectionDirection):
         super().__init__(length)
+        self.direction = direction
+    
+    def calc(self, prev_segment) -> None:
+        super().calc(prev_segment)
+        if self.direction == IntersectionDirection.RIGHT:
+            new_end = [self.length/2, -self.length/2, -90]
+        elif self.direction == IntersectionDirection.STRAIGHT:
+            new_end = [self.length, 0, 0]
+        else:
+            new_end = [self.length/2, self.length/2, 90]
+        self.end_coordinate_system = CartesianSystem2d(new_end[0], new_end[1], new_end[2], self.start_coordinate_system)
+        self.direction_angle += new_end[2]
+
 
 
 class ParkingArea(Straight):
@@ -415,3 +470,92 @@ class TrafficIsland(Segment):
         self.calc_lane(Side.LEFT)
         self.calc_lane(Side.RIGHT)
         self.calc_crosswalk_lines()
+
+class Clothoid(Segment):
+    def __init__(self, a: float, angle: float, angle_offset: float, direction: ClothoidDirection, type: ClothoidType):
+        super().__init__()
+        self.a = a
+        self.angle = angle
+        self.angle_offset = angle_offset
+        self.direction = direction
+        self.type = type
+        self.lines: List[Polygon] = []
+        self.background_polygon: List[Polygon] = []
+    
+    def get_int(self, number: float) -> int:
+        return int(number + 0.5 if number >= 0 else number -0.5)
+    
+    def move_point(self, point: List[float], postponmet: List[float]) -> List[float]:
+        return [point[0] + postponmet[0], point[1] + postponmet[1], point[2]]
+
+    def rotate_point(self, point: List[float], radian: float) -> List[float]:
+        return [point[0]*cos(radian)+point[1]*sin(radian), -point[0]*sin(radian)+point[1]*cos(radian), point[2]]
+
+    def get_clothoid_point(self, length: float, direction: float) -> List[List[float]]:
+        toggle = 1
+        x = 0
+        y = 0 
+        for loops in range(20):
+            x_ = toggle*length**(1+4*loops)/(self.a**(4*loops)*factorial(2*loops)*(1+4*loops)*2**(2*loops))
+            y_ = toggle*length**(3+4*loops)/(self.a**(2+4*loops)*factorial(1+2*loops)*(3+4*loops)*2**(1+2*loops))
+            x += x_
+            y += y_
+            toggle *= -1
+        return [x ,y*direction, length]
+
+    def get_inverted_points(self, points: List[List[float]]) -> List[List[float]]:
+        angle = radians(self.angle) * self.direction.value
+        new_points = []
+        start = points[0]
+        end = self.move_point(points[-1], [-start[0], -start[1]])
+        for point in points:
+            point = self.move_point(point, [-start[0], -start[1]])
+            point = [(-point[0]+end[0]), (point[1]-end[1]), point[2]]
+            point = self.rotate_point(point, angle)
+            new_points.append(self.move_point(point, [start[0], start[1]]))
+        return new_points[::-1]
+
+    def get_clothoid(self) -> List[List[float]]:
+        direction = self.direction.value * -1
+        arc_length_start = self.a*sqrt(2*radians(self.angle_offset))
+        arc_length_end = self.a*sqrt(2*radians(self.angle_offset + self.angle))
+        points = []
+        distance = 0.04
+        l = [i*distance for i in range(self.get_int(arc_length_start/distance), self.get_int(arc_length_end/distance)+1)]
+        l.append(arc_length_end)
+        for i in l:
+            points.append(self.get_clothoid_point(i, direction))
+        first_point = [points[0][0]*-1, points[0][1]*-1]
+        for i, point in enumerate(points):
+            points[i] = self.rotate_point(self.move_point(point, first_point), radians(self.angle_offset)*direction)
+        return points
+    
+    def get_moved_clothoid(self, points: List[List[float]], offset: float) -> List[List[float]]:
+        new_points = []
+        for point in points:
+            angle = (point[2]**2/(2*self.a**2) - radians(self.angle_offset)) * self.direction.value
+            new_points.append([point[0] + offset*sin(angle), point[1] + offset*cos(angle), point[2]])
+        return new_points
+
+    def calc(self, prev_segment: Segment) -> None:
+        super().calc(prev_segment)
+        direction = self.direction.value * -1
+        self.direction_angle += self.angle * direction
+
+        middle_points = self.get_clothoid()
+        left_lane_points = self.get_moved_clothoid(middle_points, +LINE_OFFSET)
+        right_line_points = self.get_moved_clothoid(middle_points, -LINE_OFFSET)
+
+        if self.type == ClothoidType.OPEND:
+            middle_points = self.get_inverted_points(middle_points)
+            left_lane_points = self.get_inverted_points(left_lane_points)
+            right_line_points = self.get_inverted_points(right_line_points)
+        
+        self.end_coordinate_system = CartesianSystem2d(middle_points[-1][0], middle_points[-1][1], self.angle * direction, self.start_coordinate_system)
+
+        self.lines.append([Point2d(p[0],p[1], self.start_coordinate_system) for p in middle_points])
+        self.lines.append([Point2d(p[0],p[1], self.start_coordinate_system) for p in left_lane_points])
+        self.lines.append([Point2d(p[0],p[1], self.start_coordinate_system) for p in right_line_points])
+
+        background_polygon = right_line_points + left_lane_points[::-1]
+        self.background_polygon = [Point2d(p[0],p[1], self.start_coordinate_system) for p in background_polygon]
